#! perl

use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin/../lib";

require('apt-cacher.pl');

our $cfg = read_config('/etc/apt-cacher/apt-cacher.conf');

print get_namespace(get_original_url('debian_dists_stable-updates_main_source_Sources.gz')) . "\n";
