#! perl

use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin/../lib";

use AptCacherDB;

require('apt-cacher.pl');

our $cfg = read_config('/etc/apt-cacher/apt-cacher.conf');

my $db=AptCacherDB->new();

sleep 1;
