#!/usr/bin/perl

use strict;
use warnings;

use WWW::Curl::Easy;

my $curl = WWW::Curl::Easy->new;
$curl->setopt(CURLOPT_VERBOSE, 1);
$curl->setopt(CURLOPT_FILETIME, 1);
$curl->setopt(CURLOPT_FTP_FILEMETHOD, 2); # CURLFTPMETHOD_NOCWD
$curl->setopt(CURLOPT_FTPPORT, '-');
$curl->setopt(CURLOPT_FTP_USE_EPSV, 1);
$curl->setopt(CURLOPT_URL, 'ftp://ftp.uk.debian.org/debian/dists/stable/Release.gpg');

$curl->perform;

