#!/usr/bin/perl

use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin/../lib";

require('apt-cacher.pl');

my %h;

extract_sums($_, undef, \%h) foreach @ARGV;

exit;
print "$_\n" foreach keys %h;


foreach my $file (keys %h) {
    print "$file:\n";
    my $href = hashify(\$h{$file});
    foreach (keys %$href) {
	print " $_: $href->{$_}\n" if $href->{$_};
    }
}
