#! /usr/bin/perl

use BerkeleyDB;

while (1) {

    alarm(1);
    eval {
	local $SIG{ALRM} = sub { die "Timeout\n";};
	my $status = BerkeleyDB::db_verify
	  -Filename => '/var/cache/apt-cacher/sums.db';
	print "Returned ".$status." \n";
    };
    print "$@";
}
