#!/usr/bin/perl

use strict;
use warnings;

use File::Spec;
use lib (File::Spec->splitpath($0))[1] .'../lib';
use CachedRequest;
use HTTP::Request;

require('/usr/src/apt-cacher/src/lib/apt-cacher.pl');

our $cfg = read_config('/etc/apt-cacher/apt-cacher.conf');

my $r = CachedRequest->new(HTTP::Request->new('GET' => 'http://debian/dists/stable/Release'));

sleep 1;
